;
; A few macors for debugging scheme programs...
;

(provide 'debug)

(define **debug** 1)
(define **test** #t)

;;;;;;;;;;;;;;;;;;;;; Error messages and error handling ;;;;;;;;;;;;;;;;;;;;;


(define (*****) (error "Ran into an implemented object"))

(define (throw-mismatch) (error "Mismatch failure"))

(define (assert predicate object . rest)
  (if (predicate object)
      object
      (let ((errmsg (if (pair? rest) (car rest) "Object failed assertion :")))
	(error errmsg predicate object))))

;example use:
; (assert zero? 1 "Go you do see !")

; (print "Done loading lib.scm")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define error-output-port (current-output-port))

(define (error-display s)
   (display s error-output-port))

(define (debug-display s)
   (if **debug** (error-display s)))

(define (error . rest)
   (error-display "Error: ")
   (write rest error-output-port)
   (error-display "\n")
   (sys::reset))


(define (DBG f . args)
  (display "show-apply: ")
  (display (cons f args))
  (let ((res (apply f args)))
    (display " => ")
    (display res)
    (newline)
    res))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (*test-on*)  (set! **test** #t))
(define (*test-off*) (set! **test** #f))
(define-macro (*test* . body)
   (let ((test1
	  (lambda (sexp)
	    `(begin
	      (display      "Compute:  ")
	      (write ',(car sexp))
	      (newline)
	      (let ((**expected** ',(cadr sexp))
		    (**result** ,(car sexp)))
	      (if (equal? **result** **expected**)
		  (begin
		   (display "Result Ok: ")
		   (write **result**))
		  (begin
		   (display "Bad!\007\nExpected:  ")
		   (write **expected**)
		   (newline)
		   (display "Found:     ")
		   (write **result**)))
	      (newline)
	      (newline))))))
   (if **test**
       `(begin ,@(map test1 body))
       '#f)))

;(*test* ((+ 2 2) 4))
