;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;; Scheme support macros for Tunes ;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Works with Elk and STk
; macros are an-hygienic source-to-source functions

(provide 'macrolib)
(require 'lib)


;
; Very useful: a map function that works with macros...
;
(define-macro (map-macro form mac . lst)
   (if (null? lst) form
      `(map-macro ,(append form `((,mac ,(car lst)))) ,mac ,@(cdr lst))))
; Example:
; (map-macro (begin) print 1 2 3 4 5)

;
; do something with both an source expression and its value...
;
(define-macro (fun-sym-val fun name)
      `(,fun ',name ,name))

;
; Curryfication for macros...
;
(define-macro (curry-macro mac . args)
   `(macro x `(,,mac ,,@args ,@x)))

;
; Comments
;
(define-macro (Comment XXX)
  ;(Annotate! (Current-Object) 'Comment XXX)
  '#f)


;
; Unquote these to define all the cxxxr words
; if your Scheme kernel doesn't do it for you...
;
'(define-macro (def-cxxxr string)
   (let (((cxr-name x) (make-symbol "c" x "r"))
         ((cxr-fun x l arg)
	    (if (eqv? l (string-length x)) arg
	       `( ,(cxr-name (string-ref x l)) ,(cxr-fun x (1+ l) arg)))))
    `(define (,(cxr-name string) x) ,(cxr-fun string 0 'x))))
'(map-macro (begin) def-cxxxr
   "aa" "aaa" "aaaa" "daaa"
        "daa" "adaa" "ddaa"
   "da" "ada" "aada" "dada"
        "dda" "adda" "ddda"
   "ad" "aad" "aaad" "daad"
        "dad" "adad" "ddad"
   "dd" "add" "aadd" "dadd"
        "ddd" "addd" "dddd")

;
; Tracing a function
;
(define troff-tag (cons 'trace 'off))
(define-macro (traceon fun . noreentrant)
  (let ((orig-fun (make-symbol "**orig-" fun "**")))
  `(let ((orig-fun  ,fun)
	 (noreentrant ,(not (null? noreentrant)))
	 (entered   #f))
     (set! ,fun
	   (lambda args
	      (if (and (pair? args) (eq? (car args) troff-tag))
		  (set! ,fun orig-fun)
		  (if (and noreentrant entered)
		      (apply orig-fun args)
		      (fluid-let ((entered #t))
			(format #t "TRON: ~a\n" (cons ',fun args))
			(let ((res (apply orig-fun args)))
			  (format #t "TRES=> ~a\n" res)
			  res)))))))))
(define (traceoff fun) (fun troff-tag))
