; Module for pattern management

(provide 'pattern)
(require 'lib)


(Comment "

Single-minded Patterns follow the following syntax:
variable
(pattern-expression arguments)

A pattern expression is in full control of its arguments,
but generally, this would consist in matching them one by one
against

the pattern-expression is used to dispatch
the right constructor or destructor


* When inverting a non-surjective pattern,
 it is easy to raise an exception if the pattern is non-inversible
 (assuming that's detectable in a reasonable amount of time!).
* However, when inverting a non-injective pattern,
 what value should be given ?
 = Does the program mean that any matching value should be given,
 and that the program should respect its specification whatever
 value is given ?
 = Should there be backtracking and/or parallel evaluation of
 the possibilities?
 = Should the answer stay unspecified among possible answers?
   Now, if we allow any kind of non-determinism in inverted patterns,
we could as well allow the same kind of non-determinism among patterns.
   It seems to me that the semantically correct way of doing things,
is that a match operation should indeterministically return any matching
term if possible; but because the match pattern need not be linear,
solving indeterminacies cannot be done in a local way, and sometimes,
no computable mechanism can even solve them; thus, reflective escape to
tactics that control this unification are needed.
   Anyway, let us first focus on simple patterns, and pattern language...


; arguments:
;

; Creating patterns:
(pattern (pattern-name
	  ))




; constructing a pattern:
(


; (construct pattern)
; (destruct pattern arguments)




; A constructor:
(lambda (environment parameters)
    ...
    (new-environment))

");"
;)))))))



; Representational Constructors/Destructors
(define-structure rcd-pattern
   ((lambda (environment parameters) object) constructor)
   ((lambda (environment object) parameters) destructor))



; A mark to recognize patterns ?
;(define pattern-mark (cons 'pattern pattern-mark))


(define (match-var env var value ok err)
   (lookup-env env var
      (lambda (old-value) (if (eqv? value old-value) (ok env) (err)))
      (lambda () (ok (cons-env env var val)))))

(define (match-list env list-pattern list ok err)
   (cond
      ((null? list-pattern)
          (if (null? list) (ok env) (err)))
      ((symbol? list-pattern)
          (match-var env list-pattern list ok err))
      ((pair? list-pattern)
          (if (pair? list)
	     (match env (car list-pattern) (car list)
	        (lambda (env)
	           (match-list env (cdr list-pattern) (cdr list) ok err))
	        err)
	     (err)))
      (else err)))

(define (match-constr constr len)
   (lambda (env pattern-arg arg ok err)
      (if (not (and
	     (list? arg)
	     (eqv? (length arg) (len))
	     (eqv? (car arg) constr)))
	 (err)
	 (match-list env pattern-arg arg ok err))))
	 

(define (make-constr constr arg-pattern)
   (lambda (env arg ok err)
      (match-list arg-pattern arg



(define-pattern cons
   (lambda->pattern
      (lambda (x y) ((using scheme cons) x y))
      (lambda ((scheme-object z))
         (if (pair? z) (return (car z) (cdr z)) (fail-match)))))

(define-pattern conscons
   (symmetric-pattern (x y z)
      (cons x (cons y z)))

(define-pattern (<< n
  (Make-Object
  	  (Type: pattern)
  	  (cons: (lambda ((integer x) (positive-integer y))
	            ((using scheme-support <<) x y)))
	  (des:  (lambda ((integer z))
	            (and-not ((using scheme-support <<) -1


(lambda (x y z) (list z x y))

(match (x y z)
