;;;;
;;;; t a g s . s c m            -- Unique tags for a poor man's
;;;;                               user-defined types.
;;;;
;;;; Copylefted under the GPL
;;;;
;;;;           Author: Francois-Rene Rideau [rideau@unice.fr]

(provide 'tags)

;;;;
;;;; Exported symbols
;;;; ----------------
;;;;   make-tag
;;;;   tagged-object-maker
;;;;   tagged-object-detector
;;;; see documentation strings.
;;;;
;;;;
;;;; Redefined symbols
;;;; -----------------
;;;;   pair?
;;;;
;;;; Internal symbols
;;;; ----------------
;;;;   tags::magic-tag
;;;;   tags::pair?
;;;;   tags::is-tag?
;;;;   tags::tagged-object?


;;;; Initial internal definitions

(define tags::magic-tag (list 'magic))
(define tags::pair? pair?)


;;;; main exported functions

;;;;
(define (make-tag . args)

  "creates a new, unique, tag, recognizable as such.
   as the cons of a magic tag and the given arguments"

  (cons tags::magic-tag args))


;;;;
(define (tagged-object-maker tag)

  "create a maker function for a poor man's user-type implemented as
  the cons of the type's tag and the list of initializing arguments"

  (lambda rest (cons tag rest)))


;;;;
(define (tagged-object-detector tag)

  "create a detector function for a poor man's user-type implemented as
  the cons of the type's tag and the list of initializing arguments"

  (lambda (obj) (and (tags::pair? obj) (eq? (car obj) tag))))



;;;; Redefine pair? to not see tagged objects...

(define tags::is-tag?
  (tagged-object-detector tags::magic-tag))
(define (tags::tagged-object? x)
  (and (tags::pair? x) (tags::is-tag? (car x))))
(define (pair? x)
  (and (tags::pair? x) (not (tags::tagged-object? x))))

; Note:
;
; We could similarly redefine car, cdr, etc, to check for a pair?,
; providing special accessors for tagged object,
; but this would only be boring and slowing everything down even more...
;
