#!/bin/sh
# This shell'd better have shell functions.
# If not, use zsh -f or GNU bash: that's free


### Verify that we are running from the right directory
[ -x ./autoconf.sh ] ||
{ echo "You should run this program from the autoconf/ directory" ;
  exit 1
}


### Messages ###
OKTEXT='
   Everything *looks* OK to me.
   If something isn'"'"'t really, you'"'"'ll have to manually configure Tunes.
'
HELPTEXT='
   If you manage getting Tunes to run on your system,
please send us (mailto:rideau@ens.fr) a description of how we could
autodetect and autoconfigure Tunes for it, so that next time you try,
automatic installation will succeed.
'

AUTODETECTFAILED='
   We could not autodetect what kind of system you have.
Define the OSTYPE and/or ARCH environment variables and
try autoconfiguring again.
'

UNKNOWNSYS='Unknown system. You must manually configure Tunes.
'

MAKEFILE_CONFIG_HEADER='#
# This is your local Tunes/O'"'"'TOP Makefile configuration.
# If the autoconfiguration tool didn'"'"'t get something right,
# here is precisely where you should edit it.
# and of course, it would be nice if you could send us some
# information about how we could get it right for next release.
#'


### Function library ###
CONFIG_i386 () {
	CONFIG_DEP="gcc.scm gcc-i386.scm $CONFIG_DEP"
	CC=gcc
	CFLAGS="-O2 -fomit-frame-pointer -Wall"
}
CONFIG_Linux () {
	CONFIG_DEP="linux.scm $CONFIG_DEP"
	SCHEME="elk"
	CP="/bin/cp -f"
	RM="/bin/rm -f"
	INSTALL_DIR="install -d -m 755"
	INSTALL_BIN="install -s -m 755"
	SHELL=/bin/sh
}
DEFINE () {
  echo "# $1 $2"
  eval "VAL=\$$1"
  if [ -n "$VAL" ] ; then
    echo "I found that $1=$VAL" >&2
    echo "$1 = $VAL"
  fi
  echo
}

### Initializing some variables ###
CONFIG_DEP=
CONFIG_FLAGS=

### Getting some system identifiers ###

if [ -n "$OSTYPE" ] ; then
  # The user gave us the type of the OS
  : some shells hate empty lists
elif [ -x /bin/uname ] ; then
  # BSD-style system identification
  OSTYPE="$(/bin/uname)"
else
  echo "$AUTODETECTFAILED$HELPTEXT"
  exit 2
fi

if [ -n "$ARCH" ] ; then
  # The user gave us the type of the OS
  : some shells hate empty lists
elif [ -x /bin/arch ] ; then
  # BSD-style system identification
  ARCH="$(/bin/arch)"
else
  echo "$AUTODETECTFAILED$HELPTEXT"
  exit 2
fi


### Parsing the identification string for useful information ###

case "$OSTYPE:$ARCH" in
  [Ll]inux*:i?86)
	CONFIG=linux-i386.scm
	CONFIG_Linux
	CONFIG_i386
	#CONFIG_ncurses		# should find & configure ncurses
	#CONFIG_X11		# should find & configure X11
	#CONFIG_SVGALIB		# should find & configure SVGALIB
	;;
#  sunos4:sun4)
#	CONFIG_sunos4
#	CONFIG_sun4
#	;;
  *)
	echo "$UNKNOWNSYS$HELPTEXT"
	exit 2
	;;
esac

{
echo "$MAKEFILE_CONFIG_HEADER"
DEFINE OSTYPE	"defines the type of the underlying operating system"
DEFINE ARCH	"defines the type of the hardware architecture (CPU, etc)"
DEFINE CONFIG	"is the name of the main configuration-dependent file"
DEFINE CONFIG_DEP	"defines all additional dependencies for the O'TOP"
DEFINE CONFIG_FLAGS	"defines site-specific configuration flags"
DEFINE SHELL	"is the path for the (Bourne) shell"
DEFINE SCHEME	"is the path for a scheme interpreter"
DEFINE CC	"is the path to access the C compiler"
DEFINE CFLAGS	"is the flags for the C compiler"
DEFINE CP	"is a command to copy files"
DEFINE RM	"is a command to delete files"
DEFINE INSTALL_DIR	"is a command to install directories"
DEFINE INSTALL_BIN	"is a command to install executable files"
} > Makefile.config

echo "$OKTEXT$HELPTEXT"
